@@ -7,13 +7,13 @@
  * This is free software; see Copyright file in the source
  * distribution for preciese wording.
  *
- * Copyright (C) 2002-2016 Aleksey Sanin <aleksey@aleksey.com>. All Rights Reserved.
+ * Copyright (C) 2002-2024 Aleksey Sanin <aleksey@aleksey.com>. All Rights Reserved.
  */
 #ifndef __XMLSEC_PRIVATE_H__
 #define __XMLSEC_PRIVATE_H__
 
 #ifndef XMLSEC_PRIVATE
-#error "xmlsec/private.h file contains private xmlsec definitions and should not be used outside xmlsec or xmlsec-$crypto libraries"
+#error "this file contains private xmlsec definitions and should not be used outside xmlsec or xmlsec-$crypto libraries"
 #endif /* XMLSEC_PRIVATE */
 
 #include <libxml/tree.h>
@@ -28,12 +28,13 @@
 #ifdef __GNUC__
 #ifdef HAVE_ANSIDECL_H
 #include <ansidecl.h>
-#endif
-#endif
+#endif /* HAVE_ANSIDECL_H */
+#endif /* __GNUC__ */
 
-#ifdef WIN32
+/* This is needed for UNREFERENCED_PARAMETER */
+#if defined(XMLSEC_WINDOWS)
 #include <windows.h>
-#endif
+#endif /* defined(XMLSEC_WINDOWS) */
 
 #ifdef __cplusplus
 extern "C" {
@@ -168,6 +169,23 @@ typedef int                     (*xmlSecCryptoAppDefaultKeysMngrInitMethod)
 typedef int                     (*xmlSecCryptoAppDefaultKeysMngrAdoptKeyMethod)
                                                                         (xmlSecKeysMngrPtr mngr,
                                                                          xmlSecKeyPtr key);
+/**
+ * xmlSecCryptoAppDefaultKeysMngVerifyKeyMethod:
+ * @mngr:               the pointer to keys manager.
+ * @key:                the pointer to key.
+ * @keyInfoCtx:         the key info context for verification.
+ *
+ * Verifies @key with the keys manager @mngr created with #xmlSecCryptoAppDefaultKeysMngrInit
+ * function:
+ * - Checks that key certificate is present
+ * - Checks that key certificate is valid
+ *
+ * Returns: 1 if key is verified, 0 otherwise, or a negative value if an error occurs.
+ */
+typedef int                     (*xmlSecCryptoAppDefaultKeysMngVerifyKeyMethod)
+                                                                        (xmlSecKeysMngrPtr mngr,
+                                                                         xmlSecKeyPtr key,
+                                                                         xmlSecKeyInfoCtxPtr keyInfoCtx);
 /**
  * xmlSecCryptoAppDefaultKeysMngrLoadMethod:
  * @mngr:               the pointer to keys manager.
@@ -215,9 +233,9 @@ typedef int                     (*xmlSecCryptoAppKeysMngrCertLoadMethod)(xmlSecK
 /**
  * xmlSecCryptoAppKeysMngrCertLoadMemoryMethod:
  * @mngr:               the keys manager.
- * @data:               the key data.
- * @dataSize:           the key data size.
- * @format:             the certificate format.
+ * @data:               the certificate data.
+ * @dataSize:           the certificate data size.
+ * @format:             the certificate data format.
  * @type:               the flag that indicates is the certificate in @data
  *                      trusted or not.
  *
@@ -231,6 +249,35 @@ typedef int                     (*xmlSecCryptoAppKeysMngrCertLoadMemoryMethod)(x
                                                                          xmlSecSize dataSize,
                                                                          xmlSecKeyDataFormat format,
                                                                          xmlSecKeyDataType type);
+/**
+ * xmlSecCryptoAppKeysMngrCrlLoadMethod:
+ * @mngr:               the keys manager.
+ * @filename:           the CRL file.
+ * @format:             the CRL file format.
+ *
+ * Reads crls from @filename and adds to the list of crls in @store.
+ *
+ * Returns: 0 on success or a negative value otherwise.
+ */
+typedef int                     (*xmlSecCryptoAppKeysMngrCrlLoadMethod)(xmlSecKeysMngrPtr mngr,
+                                                                         const char *filename,
+                                                                         xmlSecKeyDataFormat format);
+/**
+ * xmlSecCryptoAppKeysMngrCrlLoadMemoryMethod:
+ * @mngr:               the keys manager.
+ * @data:               the CRL data.
+ * @dataSize:           the CRL data size.
+ * @format:             the CRL data format.
+ *
+ * Reads crls from @data and adds to the list of crls in @store.
+ *
+ * Returns: 0 on success or a negative value otherwise.
+ */
+typedef int                     (*xmlSecCryptoAppKeysMngrCrlLoadMemoryMethod)(xmlSecKeysMngrPtr mngr,
+                                                                         const xmlSecByte* data,
+                                                                         xmlSecSize dataSize,
+                                                                         xmlSecKeyDataFormat format);
+
 /**
  * xmlSecCryptoAppKeyLoadMethod:
  * @filename:           the key filename.
@@ -249,6 +296,26 @@ typedef xmlSecKeyPtr            (*xmlSecCryptoAppKeyLoadMethod)         (const c
                                                                          void* pwdCallback,
                                                                          void* pwdCallbackCtx);
 
+/**
+ * xmlSecCryptoAppKeyLoadExMethod:
+ * @filename:           the key filename.
+ * @type:               the expected key type.
+ * @format:             the key file format.
+ * @pwd:                the key file password.
+ * @pwdCallback:        the key password callback.
+ * @pwdCallbackCtx:     the user context for password callback.
+ *
+ * Reads key from the a file.
+ *
+ * Returns: pointer to the key or NULL if an error occurs.
+ */
+typedef xmlSecKeyPtr            (*xmlSecCryptoAppKeyLoadExMethod)       (const char *filename,
+                                                                         xmlSecKeyDataType type,
+                                                                         xmlSecKeyDataFormat format,
+                                                                         const char *pwd,
+                                                                         void* pwdCallback,
+                                                                         void* pwdCallbackCtx);
+
 /**
  * xmlSecCryptoAppKeyLoadMemoryMethod:
  * @data:               the key data.
@@ -278,7 +345,7 @@ typedef xmlSecKeyPtr            (*xmlSecCryptoAppKeyLoadMemoryMethod)   (const x
  * @pwdCallbackCtx:     the user context for password callback.
  *
  * Reads key and all associated certificates from the PKCS12 file.
- * For uniformity, call xmlSecCryptoAppKeyLoad instead of this function. Pass
+ * For uniformity, call xmlSecCryptoAppKeyLoadEx instead of this function. Pass
  * in format=xmlSecKeyDataFormatPkcs12.
  *
  * Returns: pointer to the key or NULL if an error occurs.
@@ -296,7 +363,7 @@ typedef xmlSecKeyPtr            (*xmlSecCryptoAppPkcs12LoadMethod)      (const c
  * @pwdCallbackCtx:     the user context for password callback.
  *
  * Reads key and all associated certificates from the PKCS12 binary data.
- * For uniformity, call xmlSecCryptoAppKeyLoad instead of this function. Pass
+ * For uniformity, call xmlSecCryptoAppKeyLoadEx instead of this function. Pass
  * in format=xmlSecKeyDataFormatPkcs12.
  *
  * Returns: pointer to the key or NULL if an error occurs.
@@ -341,35 +408,47 @@ typedef int                     (*xmlSecCryptoAppKeyCertLoadMemoryMethod)(xmlSec
  * @cryptoShutdown:             the xmlsec-crypto library shutdown method.
  * @cryptoKeysMngrInit:         the xmlsec-crypto library keys manager init method.
  * @keyDataAesGetKlass:         the method to get pointer to AES key data klass.
+ * @keyDataConcatKdfGetKlass:   the method to get pointer to ConcatKDF key data klass.
  * @keyDataDesGetKlass:         the method to get pointer to DES key data klass.
+ * @keyDataDhGetKlass:          the method to get pointer to DH key data klass.
  * @keyDataDsaGetKlass:         the method to get pointer to DSA key data klass.
- * @keyDataEcdsaGetKlass:       the method to get pointer to ECDSA key data klass.
+ * @keyDataEcGetKlass:          the method to get pointer to EC key data klass.
  * @keyDataGost2001GetKlass:    the method to get pointer to GOST 2001 key data klass.
  * @keyDataGostR3410_2012_256GetKlass: the method to get pointer to GOST R 34.10-2012 256 bit key data klass.
  * @keyDataGostR3410_2012_512GetKlass: the method to get pointer to GOST R 34.10-2012 512 bit key data klass.
  * @keyDataHmacGetKlass:        the method to get pointer to HMAC key data klass.
+ * @keyDataPbkdf2GetKlass:      the method to get pointer to PBKDF2 key data klass.
  * @keyDataRsaGetKlass:         the method to get pointer to RSA key data klass.
  * @keyDataX509GetKlass:        the method to get pointer to X509 key data klass.
  * @keyDataRawX509CertGetKlass: the method to get pointer to raw X509 cert key data klass.
  * @x509StoreGetKlass:          the method to get pointer to X509 key data store.
+ * @keyDataDEREncodedKeyValueGetKlass:        the method to get pointer to X509 key data klass.
  * @transformAes128CbcGetKlass: the method to get pointer to AES 128 CBC encryption transform.
  * @transformAes192CbcGetKlass: the method to get pointer to AES 192 CBC encryption transform.
  * @transformAes256CbcGetKlass: the method to get pointer to AES 256 CBC encryption transform.
  * @transformAes128GcmGetKlass: the method to get pointer to AES 128 GCM encryption transform.
  * @transformAes192GcmGetKlass: the method to get pointer to AES 192 GCM encryption transform.
  * @transformAes256GcmGetKlass: the method to get pointer to AES 256 GCM encryption transform.
+ * @transformConcatKdfGetKlass: the method to get pointer to PBKDF2 KDF transform.
  * @transformKWAes128GetKlass:  the method to get pointer to AES 128 key wrapper transform.
  * @transformKWAes192GetKlass:  the method to get pointer to AES 192 key wrapper transform.
  * @transformKWAes256GetKlass:  the method to get pointer to AES 256 key wrapper transform.
  * @transformDes3CbcGetKlass:   the method to get pointer to Triple DES encryption transform.
  * @transformKWDes3GetKlass:    the method to get pointer to Triple DES key wrapper transform.
+ * @transformDhEsGetKlass:      the method to get pointer to DH-ES key agreement transform.
  * @transformDsaSha1GetKlass:   the method to get pointer to DSA-SHA1 signature transform.
- * @transformDsaSha256GetKlass: the method to get pointer to DSA-SHA256 signature transform.
+ * @transformDsaSha256GetKlass: the method to get pointer to DSA-SHA2-256 signature transform.
+ * @transformEcdhGetKlass:      the method to get pointer to ECDH key agreement transform.
+ * @transformEcdsaRipemd160GetKlass: the method to get pointer to ECDSA-RIPEMD160 signature transform.
  * @transformEcdsaSha1GetKlass: the method to get pointer to ECDSA-SHA1 signature transform.
- * @transformEcdsaSha224GetKlass: the method to get pointer to ECDSA-SHA224 signature transform.
- * @transformEcdsaSha256GetKlass: the method to get pointer to ECDSA-SHA256 signature transform.
- * @transformEcdsaSha384GetKlass: the method to get pointer to ECDSA-SHA384 signature transform.
- * @transformEcdsaSha512GetKlass: the method to get pointer to ECDSA-SHA512 signature transform.
+ * @transformEcdsaSha224GetKlass: the method to get pointer to ECDSA-SHA2-224 signature transform.
+ * @transformEcdsaSha256GetKlass: the method to get pointer to ECDSA-SHA2-256 signature transform.
+ * @transformEcdsaSha384GetKlass: the method to get pointer to ECDSA-SHA2-384 signature transform.
+ * @transformEcdsaSha512GetKlass: the method to get pointer to ECDSA-SHA2-512 signature transform.
+ * @transformEcdsaSha3_224GetKlass: the method to get pointer to ECDSA-SHA3-224 signature transform.
+ * @transformEcdsaSha3_256GetKlass: the method to get pointer to ECDSA-SHA3-256 signature transform.
+ * @transformEcdsaSha3_384GetKlass: the method to get pointer to ECDSA-SHA3-384 signature transform.
+ * @transformEcdsaSha3_512GetKlass: the method to get pointer to ECDSA-SHA3-512 signature transform.
  * @transformGost2001GostR3411_94GetKlass: the method to get pointer to GOST2001 transform.
  * @transformGostR3410_2012GostR3411_2012_256GetKlass: the method to get pointer to GOST R 34.10-2012 - GOST R 34.11-2012 256bit transform.
  * @transformGostR3410_2012GostR3411_2012_512GetKlass: the method to get pointer to GOST R 34.10-2012 - GOST R 34.11_2012 512bit transform.
@@ -381,33 +460,52 @@ typedef int                     (*xmlSecCryptoAppKeyCertLoadMemoryMethod)(xmlSec
  * @transformHmacSha384GetKlass: the method to get pointer to HMAC-SHA384 transform.
  * @transformHmacSha512GetKlass: the method to get pointer to HMAC-SHA512 transform.
  * @transformMd5GetKlass:       the method to get pointer to MD5 digest transform.
+ * @transformPbkdf2GetKlass: the method to get pointer to Pbkdf2 KDF transform.
  * @transformRipemd160GetKlass: the method to get pointer to RIPEMD160 digest transform.
  * @transformRsaMd5GetKlass:    the method to get pointer to RSA-MD5 signature transform.
  * @transformRsaRipemd160GetKlass: the method to get pointer to RSA-RIPEMD160 signature transform.
  * @transformRsaSha1GetKlass:   the method to get pointer to RSA-SHA1 signature transform.
- * @transformRsaSha224GetKlass: the method to get pointer to RSA-SHA224 signature transform.
- * @transformRsaSha256GetKlass: the method to get pointer to RSA-SHA256 signature transform.
- * @transformRsaSha384GetKlass: the method to get pointer to RSA-SHA384 signature transform.
- * @transformRsaSha512GetKlass: the method to get pointer to RSA-SHA512 signature transform.
+ * @transformRsaSha224GetKlass: the method to get pointer to RSA-SHA2-224 signature transform.
+ * @transformRsaSha256GetKlass: the method to get pointer to RSA-SHA2-256 signature transform.
+ * @transformRsaSha384GetKlass: the method to get pointer to RSA-SHA2-384 signature transform.
+ * @transformRsaSha512GetKlass: the method to get pointer to RSA-SHA2-512 signature transform.
+ * @transformRsaPssSha1GetKlass:   the method to get pointer to RSA-PSS-HA1 signature transform.
+ * @transformRsaPssSha224GetKlass: the method to get pointer to RSA-PSS-SHA2-224 signature transform.
+ * @transformRsaPssSha256GetKlass: the method to get pointer to RSA-PSS-SHA2-256 signature transform.
+ * @transformRsaPssSha384GetKlass: the method to get pointer to RSA-PSS-SHA2-384 signature transform.
+ * @transformRsaPssSha512GetKlass: the method to get pointer to RSA-PSS-SHA2-512 signature transform.
+ * @transformRsaPssSha3_224GetKlass: the method to get pointer to RSA-PSS-SHA2-224 signature transform.
+ * @transformRsaPssSha3_256GetKlass: the method to get pointer to RSA-PSS-SHA2-256 signature transform.
+ * @transformRsaPssSha3_384GetKlass: the method to get pointer to RSA-PSS-SHA2-384 signature transform.
+ * @transformRsaPssSha3_512GetKlass: the method to get pointer to RSA-PSS-SHA2-512 signature transform.
  * @transformRsaPkcs1GetKlass:  the method to get pointer to RSA-PKCS1_5 key transport transform.
- * @transformRsaOaepGetKlass:   the method to get pointer to RSA-OAEP key transport transform.
+ * @transformRsaOaepGetKlass:   the method to get pointer to RSA-OAEP key transport transform (XMLEnc 1.0).
+ * @transformRsaOaepEnc11GetKlass: the method to get pointer to RSA-OAEP key transport transform (XMLEnc 1.1).
  * @transformGostR3411_94GetKlass: the method to get pointer to GOST R3411 transform.
  * @transformGostR3411_2012_256GetKlass: the method to get pointer to GOST R 34.11-2012 256 bit transform.
  * @transformGostR3411_2012_512GetKlass: the method to get pointer to GOST R 34.11-2012 512 bit transform.
  * @transformSha1GetKlass:      the method to get pointer to SHA1 digest transform.
- * @transformSha224GetKlass:    the method to get pointer to SHA224 digest transform.
- * @transformSha256GetKlass:    the method to get pointer to SHA256 digest transform.
- * @transformSha384GetKlass:    the method to get pointer to SHA384 digest transform.
- * @transformSha512GetKlass:    the method to get pointer to SHA512 digest transform.
+ * @transformSha224GetKlass:    the method to get pointer to SHA2-224 digest transform.
+ * @transformSha256GetKlass:    the method to get pointer to SHA2-256 digest transform.
+ * @transformSha384GetKlass:    the method to get pointer to SHA2-384 digest transform.
+ * @transformSha512GetKlass:    the method to get pointer to SHA2-512 digest transform.
+ * @transformSha3_224GetKlass:    the method to get pointer to SHA3-224 digest transform.
+ * @transformSha3_256GetKlass:    the method to get pointer to SHA3-256 digest transform.
+ * @transformSha3_384GetKlass:    the method to get pointer to SHA3-384 digest transform.
+ * @transformSha3_512GetKlass:    the method to get pointer to SHA3-512 digest transform.
  * @cryptoAppInit:              the default crypto engine initialization method.
  * @cryptoAppShutdown:          the default crypto engine shutdown method.
  * @cryptoAppDefaultKeysMngrInit:       the default keys manager init method.
  * @cryptoAppDefaultKeysMngrAdoptKey:   the default keys manager adopt key method.
+ * @cryptoAppDefaultKeysMngrVerifyKey:  the defualt keys manager verify key method.
  * @cryptoAppDefaultKeysMngrLoad:       the default keys manager load method.
  * @cryptoAppDefaultKeysMngrSave:       the default keys manager save method.
  * @cryptoAppKeysMngrCertLoad:          the default keys manager file cert load method.
  * @cryptoAppKeysMngrCertLoadMemory:    the default keys manager memory cert load method.
+ * @cryptoAppKeysMngrCrlLoad:           the default keys manager file crl load method.
+ * @cryptoAppKeysMngrCrlLoadMemory:     the default keys manager memory crl load method.
  * @cryptoAppKeyLoad:           the key file load method.
+ * @cryptoAppKeyLoadEx:         the key file load method.
  * @cryptoAppKeyLoadMemory:     the meory key load method.
  * @cryptoAppPkcs12Load:        the pkcs12 file load method.
  * @cryptoAppPkcs12LoadMemory:  the memory pkcs12 load method.
@@ -425,16 +523,20 @@ struct _xmlSecCryptoDLFunctions {
 
     /* Key data ids */
     xmlSecCryptoKeyDataGetKlassMethod            keyDataAesGetKlass;
+    xmlSecCryptoKeyDataGetKlassMethod            keyDataConcatKdfGetKlass;
     xmlSecCryptoKeyDataGetKlassMethod            keyDataDesGetKlass;
+    xmlSecCryptoKeyDataGetKlassMethod            keyDataDhGetKlass;
     xmlSecCryptoKeyDataGetKlassMethod            keyDataDsaGetKlass;
-    xmlSecCryptoKeyDataGetKlassMethod            keyDataEcdsaGetKlass;
+    xmlSecCryptoKeyDataGetKlassMethod            keyDataEcGetKlass;
     xmlSecCryptoKeyDataGetKlassMethod            keyDataGost2001GetKlass;
     xmlSecCryptoKeyDataGetKlassMethod            keyDataGostR3410_2012_256GetKlass;
     xmlSecCryptoKeyDataGetKlassMethod            keyDataGostR3410_2012_512GetKlass;
     xmlSecCryptoKeyDataGetKlassMethod            keyDataHmacGetKlass;
+    xmlSecCryptoKeyDataGetKlassMethod            keyDataPbkdf2GetKlass;
     xmlSecCryptoKeyDataGetKlassMethod            keyDataRsaGetKlass;
     xmlSecCryptoKeyDataGetKlassMethod            keyDataX509GetKlass;
     xmlSecCryptoKeyDataGetKlassMethod            keyDataRawX509CertGetKlass;
+    xmlSecCryptoKeyDataGetKlassMethod            keyDataDEREncodedKeyValueGetKlass;
 
     /* Key data store ids */
     xmlSecCryptoKeyDataStoreGetKlassMethod       x509StoreGetKlass;
@@ -446,18 +548,32 @@ struct _xmlSecCryptoDLFunctions {
     xmlSecCryptoTransformGetKlassMethod          transformAes128GcmGetKlass;
     xmlSecCryptoTransformGetKlassMethod          transformAes192GcmGetKlass;
     xmlSecCryptoTransformGetKlassMethod          transformAes256GcmGetKlass;
+
     xmlSecCryptoTransformGetKlassMethod          transformKWAes128GetKlass;
     xmlSecCryptoTransformGetKlassMethod          transformKWAes192GetKlass;
     xmlSecCryptoTransformGetKlassMethod          transformKWAes256GetKlass;
+
+    xmlSecCryptoTransformGetKlassMethod          transformConcatKdfGetKlass;
+
     xmlSecCryptoTransformGetKlassMethod          transformDes3CbcGetKlass;
     xmlSecCryptoTransformGetKlassMethod          transformKWDes3GetKlass;
+
+    xmlSecCryptoTransformGetKlassMethod          transformDhEsGetKlass;
+
     xmlSecCryptoTransformGetKlassMethod          transformDsaSha1GetKlass;
     xmlSecCryptoTransformGetKlassMethod          transformDsaSha256GetKlass;
+
+    xmlSecCryptoTransformGetKlassMethod          transformEcdhGetKlass;
+    xmlSecCryptoTransformGetKlassMethod          transformEcdsaRipemd160GetKlass;
     xmlSecCryptoTransformGetKlassMethod          transformEcdsaSha1GetKlass;
     xmlSecCryptoTransformGetKlassMethod          transformEcdsaSha224GetKlass;
     xmlSecCryptoTransformGetKlassMethod          transformEcdsaSha256GetKlass;
     xmlSecCryptoTransformGetKlassMethod          transformEcdsaSha384GetKlass;
     xmlSecCryptoTransformGetKlassMethod          transformEcdsaSha512GetKlass;
+    xmlSecCryptoTransformGetKlassMethod          transformEcdsaSha3_224GetKlass;
+    xmlSecCryptoTransformGetKlassMethod          transformEcdsaSha3_256GetKlass;
+    xmlSecCryptoTransformGetKlassMethod          transformEcdsaSha3_384GetKlass;
+    xmlSecCryptoTransformGetKlassMethod          transformEcdsaSha3_512GetKlass;
     xmlSecCryptoTransformGetKlassMethod          transformGost2001GostR3411_94GetKlass;
     xmlSecCryptoTransformGetKlassMethod          transformGostR3410_2012GostR3411_2012_256GetKlass;
     xmlSecCryptoTransformGetKlassMethod          transformGostR3410_2012GostR3411_2012_512GetKlass;
@@ -469,35 +585,62 @@ struct _xmlSecCryptoDLFunctions {
     xmlSecCryptoTransformGetKlassMethod          transformHmacSha384GetKlass;
     xmlSecCryptoTransformGetKlassMethod          transformHmacSha512GetKlass;
     xmlSecCryptoTransformGetKlassMethod          transformMd5GetKlass;
+    xmlSecCryptoTransformGetKlassMethod          transformPbkdf2GetKlass;
     xmlSecCryptoTransformGetKlassMethod          transformRipemd160GetKlass;
     xmlSecCryptoTransformGetKlassMethod          transformRsaMd5GetKlass;
     xmlSecCryptoTransformGetKlassMethod          transformRsaRipemd160GetKlass;
+
     xmlSecCryptoTransformGetKlassMethod          transformRsaSha1GetKlass;
     xmlSecCryptoTransformGetKlassMethod          transformRsaSha224GetKlass;
     xmlSecCryptoTransformGetKlassMethod          transformRsaSha256GetKlass;
     xmlSecCryptoTransformGetKlassMethod          transformRsaSha384GetKlass;
     xmlSecCryptoTransformGetKlassMethod          transformRsaSha512GetKlass;
+
+    xmlSecCryptoTransformGetKlassMethod          transformRsaPssSha1GetKlass;
+
+    xmlSecCryptoTransformGetKlassMethod          transformRsaPssSha224GetKlass;
+    xmlSecCryptoTransformGetKlassMethod          transformRsaPssSha256GetKlass;
+    xmlSecCryptoTransformGetKlassMethod          transformRsaPssSha384GetKlass;
+    xmlSecCryptoTransformGetKlassMethod          transformRsaPssSha512GetKlass;
+
+    xmlSecCryptoTransformGetKlassMethod          transformRsaPssSha3_224GetKlass;
+    xmlSecCryptoTransformGetKlassMethod          transformRsaPssSha3_256GetKlass;
+    xmlSecCryptoTransformGetKlassMethod          transformRsaPssSha3_384GetKlass;
+    xmlSecCryptoTransformGetKlassMethod          transformRsaPssSha3_512GetKlass;
+
     xmlSecCryptoTransformGetKlassMethod          transformRsaPkcs1GetKlass;
     xmlSecCryptoTransformGetKlassMethod          transformRsaOaepGetKlass;
+    xmlSecCryptoTransformGetKlassMethod          transformRsaOaepEnc11GetKlass;
     xmlSecCryptoTransformGetKlassMethod          transformGostR3411_94GetKlass;
     xmlSecCryptoTransformGetKlassMethod          transformGostR3411_2012_256GetKlass;
     xmlSecCryptoTransformGetKlassMethod          transformGostR3411_2012_512GetKlass;
+
     xmlSecCryptoTransformGetKlassMethod          transformSha1GetKlass;
+
     xmlSecCryptoTransformGetKlassMethod          transformSha224GetKlass;
     xmlSecCryptoTransformGetKlassMethod          transformSha256GetKlass;
     xmlSecCryptoTransformGetKlassMethod          transformSha384GetKlass;
     xmlSecCryptoTransformGetKlassMethod          transformSha512GetKlass;
 
+    xmlSecCryptoTransformGetKlassMethod          transformSha3_224GetKlass;
+    xmlSecCryptoTransformGetKlassMethod          transformSha3_256GetKlass;
+    xmlSecCryptoTransformGetKlassMethod          transformSha3_384GetKlass;
+    xmlSecCryptoTransformGetKlassMethod          transformSha3_512GetKlass;
+
     /* High level routines form xmlsec command line utility */
     xmlSecCryptoAppInitMethod                    cryptoAppInit;
     xmlSecCryptoAppShutdownMethod                cryptoAppShutdown;
     xmlSecCryptoAppDefaultKeysMngrInitMethod     cryptoAppDefaultKeysMngrInit;
     xmlSecCryptoAppDefaultKeysMngrAdoptKeyMethod cryptoAppDefaultKeysMngrAdoptKey;
+    xmlSecCryptoAppDefaultKeysMngVerifyKeyMethod cryptoAppDefaultKeysMngrVerifyKey;
     xmlSecCryptoAppDefaultKeysMngrLoadMethod     cryptoAppDefaultKeysMngrLoad;
     xmlSecCryptoAppDefaultKeysMngrSaveMethod     cryptoAppDefaultKeysMngrSave;
     xmlSecCryptoAppKeysMngrCertLoadMethod        cryptoAppKeysMngrCertLoad;
     xmlSecCryptoAppKeysMngrCertLoadMemoryMethod  cryptoAppKeysMngrCertLoadMemory;
+    xmlSecCryptoAppKeysMngrCrlLoadMethod         cryptoAppKeysMngrCrlLoad;
+    xmlSecCryptoAppKeysMngrCrlLoadMemoryMethod   cryptoAppKeysMngrCrlLoadMemory;
     xmlSecCryptoAppKeyLoadMethod                 cryptoAppKeyLoad;
+    xmlSecCryptoAppKeyLoadExMethod               cryptoAppKeyLoadEx;
     xmlSecCryptoAppKeyLoadMemoryMethod           cryptoAppKeyLoadMemory;
     xmlSecCryptoAppPkcs12LoadMethod              cryptoAppPkcs12Load;
     xmlSecCryptoAppPkcs12LoadMemoryMethod        cryptoAppPkcs12LoadMemory;
@@ -521,11 +664,12 @@ struct _xmlSecCryptoDLFunctions {
 
 /**
  * UNREFERENCED_PARAMETER:
+ * @param:    the parameter without references.
  *
- * Macro used to signal to MSVC unused function parameters
+ * Macro used to signal unused function parameters
  */
 #ifndef UNREFERENCED_PARAMETER
-#define UNREFERENCED_PARAMETER(x)
+#define UNREFERENCED_PARAMETER(param)   ((void)(param))
 #endif /* UNREFERENCED_PARAMETER */
 
 /***********************************************************************
@@ -598,9 +742,69 @@ struct _xmlSecCryptoDLFunctions {
     xmlSecFuncToPtr_ ##func_type((func))
 
 
+/**
+ * XMLSEC_X509DATA_CERTIFICATE_NODE:
+ *
+ * &lt;dsig:X509Certificate/&gt; node found or would be written back.
+ */
+#define XMLSEC_X509DATA_CERTIFICATE_NODE                        0x00000001
+/**
+ * XMLSEC_X509DATA_SUBJECTNAME_NODE:
+ *
+ * &lt;dsig:X509SubjectName/&gt; node found or would be written back.
+ */
+#define XMLSEC_X509DATA_SUBJECTNAME_NODE                        0x00000002
+/**
+ * XMLSEC_X509DATA_ISSUERSERIAL_NODE:
+ *
+ * &lt;dsig:X509IssuerSerial/&gt; node found or would be written back.
+ */
+#define XMLSEC_X509DATA_ISSUERSERIAL_NODE                       0x00000004
+/**
+ * XMLSEC_X509DATA_SKI_NODE:
+ *
+ * &lt;dsig:X509SKI/&gt; node found or would be written back.
+ */
+#define XMLSEC_X509DATA_SKI_NODE                                0x00000008
+/**
+ * XMLSEC_X509DATA_CRL_NODE:
+ *
+ * &lt;dsig:X509CRL/&gt; node found or would be written back.
+ */
+#define XMLSEC_X509DATA_CRL_NODE                                0x00000010
+/**
+ * XMLSEC_X509DATA_DIGEST_NODE:
+ *
+ * &lt;dsig:X509Digest/&gt; node found or would be written back.
+ */
+#define XMLSEC_X509DATA_DIGEST_NODE                             0x00000020
+
+ /**
+ * XMLSEC_X509DATA_DEFAULT:
+ *
+ * Default set of nodes to write in case of empty
+ * &lt;dsig:X509Data/&gt; node template.
+ */
+#define XMLSEC_X509DATA_DEFAULT \
+        (XMLSEC_X509DATA_CERTIFICATE_NODE | XMLSEC_X509DATA_CRL_NODE)
+
+
+
+/**
+* XMLSEC_X509DATA_SHIFT_IF_NOT_EMPTY:
+*
+* Shift bits if node present but and not empty.
+*/
+#define XMLSEC_X509DATA_SHIFT_IF_NOT_EMPTY                      16
+
+
+/* helper macros */
+#define XMLSEC_X509DATA_HAS_EMPTY_NODE(content, flag)       ( ((content) & (flag)) != 0 )
+#define XMLSEC_X509DATA_HAS_NON_EMPTY_NODE(content, flag)   ( ((content) & ((flag) << XMLSEC_X509DATA_SHIFT_IF_NOT_EMPTY)) != 0 )
+#define XMLSEC_X509DATA_HAS_NODE(content, flag)             ( XMLSEC_X509DATA_HAS_EMPTY_NODE(content, flag) || XMLSEC_X509DATA_HAS_NON_EMPTY_NODE(content, flag))
+
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
 
 #endif /* __XMLSEC_PRIVATE_H__ */
-
